@@ -1,15 +1,34 @@
 package VRML::VRML1::Standard;
 
+############################## Copyright ##############################
+#								      #
+# This program is Copyright 1996,1998 by Hartmut Palm.		      #
+# This program is free software; you can redistribute it and/or	      #
+# modify it under the terms of the GNU General Public License	      #
+# as published by the Free Software Foundation; either version 2      #
+# of the License, or (at your option) any later version.	      #
+# 								      #
+# This program is distributed in the hope that it will be useful,     #
+# but WITHOUT ANY WARRANTY; without even the implied warranty of      #
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	      #
+# GNU General Public License for more details.			      #
+# 								      #
+# If you do not have a copy of the GNU General Public License write   #
+# to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     #
+# MA 02139, USA.						      #
+#								      #
+#######################################################################
+
 require 5.000;
 require VRML::Base;
 use strict;
 use vars qw(@ISA $VERSION);
 @ISA = qw(VRML::Base);
-$VERSION = "1.02";
+$VERSION = "1.03";
 
 =head1 NAME
 
-VRML::VRML1::Standard.pm - implements nodes the VRML 1.x standard
+VRML::VRML1::Standard.pm - implements the VRML 1.x standard nodes
 
 =head1 SYNOPSIS
 
@@ -926,7 +945,7 @@ sub End {
     my ($comment) = @_;
     return $self->VRML_put("# ERROR: TAB < 0 !\n") unless $self->{'TAB'};
     chop($self->{'TAB'});
-    $comment = $comment ? " # $comment" : "";
+    $comment = $comment &&  $self->{'DEBUG'} ? " # $comment" : "";
     my $vrml = $self->{'TAB'}."}$comment\n";
     push @{$self->{'VRML'}}, $vrml;
     shift @{$self->{'XYZ'}};
